/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "portal.hpp"

#include "objects.hpp"
#include "camera.hpp"

#include "macros.hpp"

#include <GL/gl.h>

#include <list>

std::list<Portal*> gPortals;

void initPortal(Portal * portal)
{
	/* empty */
}

void addPortal(Portal * portal)
{
	gPortals.push_back(portal);
}

void renderPortalsIntern(int depth, void (*drawObjects) (void))
{
	GLdouble plane[] = { 0.0, 0.0, 1.0, -0.01 };
	int clipplane = GL_CLIP_PLANE1 + depth;
	int stencilvalue = 1 << depth;
	int stencilmask = 2 * stencilvalue - 1;

	/* for all gPortals do ... */
	FOREACH(std::list<Portal*>, gPortals, run)
	{
		Portal *p1 = *run;
		Portal *p2 = p1->other;
		
		glStencilMask(stencilvalue);
		glClear(GL_STENCIL_BUFFER_BIT);
		
#if 0
		Vector3 norm = vector3(0.0f, 0.0f, 1.0f);
		float d1 = dot(norm, sub(sgCamera, p1->pos));
		float d2 = dot(sub(sgLookat, sgCamera), sub(p1->pos, sgCamera));

		if (d1 < 0.0f || d2 < 0.0f)
		{
			continue;
		}
#endif
		
		glPushMatrix();
		{
			/*
			 * transform for portals location
			 */
			glTranslatef(p1->pos.x, p1->pos.y, p1->pos.z);
			quaternionTransform(p1->orientation);
			glTranslatef(0.0f, 0.0f, 0.01f);
	
			/*
			 * only set stencil buffer
			 */
			glColorMask(0, 0, 0, 0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
			glStencilFunc(GL_ALWAYS, stencilvalue, stencilvalue);
	
			drawCircle(PORTAL_RADIUS);
	
			glColorMask(1, 1, 1, 1);
	
			/*
			 * draw again to clear depth and color buffer
			 */
			glDepthRange(1, 1);
			glDepthFunc(GL_ALWAYS);
			glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
			glStencilFunc(GL_EQUAL, stencilmask, stencilmask);
	
			glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
			drawCircle(PORTAL_RADIUS);
	
			glDepthFunc(GL_LESS);
			glDepthRange(0, 1);
	
			if (depth < 1)
			{
				/*
				 * transform for scene behind portal
				 */
				glPushMatrix();
				{
					glTranslatef(0.0f, 0.0f, -0.01f);

					glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
					
					glClipPlane(clipplane, plane);
					glEnable(clipplane);
					{
						quaternionTransform(transposeQuaternion(p2->orientation));
						glTranslatef(-p2->pos.x, -p2->pos.y, -p2->pos.z);

						/* draw scene behind portal */
						drawObjects();
						renderPortalsIntern(depth + 1, drawObjects);
						glStencilMask(stencilvalue);
					}
					glDisable(clipplane);
				}
				glPopMatrix();
			}
	
			glDepthFunc(GL_LEQUAL);
	
			/* lightup portal */
	
			glDepthFunc(GL_ALWAYS);
	
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
			glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
			glStencilFunc(GL_EQUAL, stencilmask, stencilmask);
	
			glColor4f(1.0f, 1.0f, 1.0f, 0.2f);
			drawCircle(PORTAL_RADIUS);
	
			drawRing(PORTAL_RADIUS);
	
			glDisable(GL_BLEND);
	
			/* reset stencil buffer */
	
			glDepthFunc(GL_LESS);
			glColorMask(0, 0, 0, 0);
			glDepthMask(0);
			glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
			glStencilFunc(GL_ALWAYS, 0, stencilvalue);
	
			drawCircle(PORTAL_RADIUS);
	
			glColorMask(1, 1, 1, 1);
			glDepthMask(1);
			glDepthFunc(GL_LESS);
		}
		glPopMatrix();
	}
}

void renderPortals(void (*drawObjects) (void))
{
	glStencilMask(~0);
	glClearStencil(0);
	glClear(GL_STENCIL_BUFFER_BIT);

	glEnable(GL_STENCIL_TEST);
	{
		renderPortalsIntern(0, drawObjects);
	}
	glDisable(GL_STENCIL_TEST);
}
