/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _quaternion_hpp_
#define _quaternion_hpp_

#include "vector.hpp"

struct Quaternion
{
	float s;
	Vector3 v;

	Quaternion();
	Quaternion(float s, Vector3 v);
	void operator*=(const Quaternion& other);
};

float lenQuaternion(Quaternion q);

Quaternion normQuaternion(Quaternion q);

Quaternion transposeQuaternion(Quaternion q);

Quaternion scaleQuaternion(float s, Quaternion a);

Quaternion addQuaternion(Quaternion a, Quaternion b);

Quaternion subQuaternion(Quaternion a, Quaternion b);

float dotQuaternion(Quaternion a, Quaternion b);

Quaternion mulQuaternion(Quaternion a, Quaternion b);

Quaternion mkQuaternion(float alpha, Vector3 v);

Quaternion interpolate(float t, Quaternion a, Quaternion b);

Vector3 rotateVector(const Vector3& v, const Quaternion& q);

void quaternionTransform(Quaternion q);

inline void Quaternion::operator*=(const Quaternion& other)
{
	*this = mulQuaternion(other, *this);
}

#endif
