/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _vector_hpp_
#define _vector_hpp_

#include "types.hpp"

typedef struct
{
	float x;
	float y;
} Vector2;

struct Vector3
{
	float x;
	float y;
	float z;
	
	Vector3 operator+(const Vector3& other) const;
	Vector3 operator-(const Vector3& other) const;
	void operator+=(const Vector3& other);
};

typedef struct
{
	Vector3 origin;
	Vector3 vx;
	Vector3 vy;
	Vector3 normal;
} Orientation;

Vector2 vector2(float x, float y);

Vector3 vector3(float x, float y, float z);

float len(const Vector3 v);

Vector3 norm(const Vector3 v);

Vector3 scale(float s, const Vector3 a);

Vector3 add(const Vector3 a, const Vector3 b);

Vector3 sub(const Vector3 a, const Vector3 b);

Vector3 neg(const Vector3 a);

float dot(const Vector3 a, const Vector3 b);

Vector3 cross(const Vector3 a, const Vector3 b);

Vector3 midpoint(const Vector3 * quad);

void initProjectMat(Matrix m, float fov);

inline Vector3 Vector3::operator+(const Vector3& other) const
{
	return add(*this, other);
}

inline Vector3 Vector3::operator-(const Vector3& other) const
{
	return sub(*this, other);
}

inline void Vector3::operator+=(const Vector3& other)
{
	*this = *this + other;
}

#endif
